"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DesignType_1 = require("./DesignType");
const DesignUsage_1 = require("./DesignUsage");
const QRData_1 = require("./QRData");
const ByteUtils_1 = require("./support/ByteUtils");
const ColorPalette_1 = require("./ColorPalette");
class Design {
    constructor(usage = DesignUsage_1.DesignUsage.CustomDesign) {
        /** The player ID (2 bytes). */
        this.playerId = 0;
        /** The creator's sex (1 byte). */
        this.sex = 0;
        /** The village ID (2 bytes). */
        this.villageId = 0;
        /** The language ID (1 byte). */
        this.languageId = 1;
        /** The country ID (1 byte). */
        this.countryId = 49;
        /** The region ID (1 byte). */
        this.regionId = 11;
        /** The color palette (15 bytes). */
        this.colorPalette = new ColorPalette_1.ColorPalette;
        /** The color (this field's purpose is currently unknown) (1 byte). */
        this.color = 0;
        /** The looks (this field's purpose is currently unknown) (1 byte). */
        this.looks = 0;
        /** The usage. */
        this._usage = DesignUsage_1.DesignUsage.CustomDesign;
        this._usage = usage;
        this._colorData = this.createColorData();
    }
    static fromBytes(bytes) {
        const design = new Design;
        design.loadBytes(bytes);
        return design;
    }
    static fromQRData(values) {
        const design = new Design;
        design.loadQRData(values);
        return design;
    }
    get usage() { return this._usage; }
    set usage(value) {
        this._usage = value;
        this.refreshColorData();
    }
    /** The usage ID. */
    get usageId() { return this.usage.id; }
    set usageId(value) {
        const usage = DesignUsage_1.DesignUsage.fromId(value);
        if (!usage) {
            throw Error('Invalid usage ID');
        }
        this.usage = usage;
    }
    get colorData() { return this._colorData; }
    set colorData(value) {
        if (value.length !== this._colorData.length) {
            throw new Error('Invalid data length');
        }
        this._colorData = value;
    }
    refreshColorData() {
        this._colorData = this.createColorData(this.colorData);
    }
    createColorData(oldColorData) {
        const colorData = new Uint8Array(this.usage.colorDataLength);
        if (oldColorData) {
            colorData.set(oldColorData.subarray(0, colorData.length));
        }
        return colorData;
    }
    // Export
    getBytes() {
        if (this.title === undefined) {
            throw new Error('Title is required');
        }
        if (this.creator === undefined) {
            throw new Error('Creator name is required');
        }
        if (this.village === undefined) {
            throw new Error('Village name is required');
        }
        const data = new Uint8Array(this.usage.byteLength);
        ByteUtils_1.setString(data, 0, 42, this.title);
        ByteUtils_1.setNumber(data, 42, 2, this.playerId);
        ByteUtils_1.setString(data, 44, 18, this.creator);
        ByteUtils_1.setNumber(data, 62, 1, this.sex);
        ByteUtils_1.setNumber(data, 64, 2, this.villageId);
        ByteUtils_1.setString(data, 66, 18, this.village);
        ByteUtils_1.setNumber(data, 84, 1, this.languageId);
        ByteUtils_1.setNumber(data, 86, 1, this.countryId);
        ByteUtils_1.setNumber(data, 87, 1, this.regionId);
        ByteUtils_1.setByteSubset(data, 88, this.colorPalette.bytes);
        ByteUtils_1.setNumber(data, 103, 1, this.color);
        ByteUtils_1.setNumber(data, 104, 1, this.looks);
        ByteUtils_1.setNumber(data, 105, 1, this.usageId);
        ByteUtils_1.setByteSubset(data, 108, this.colorData);
        return data;
    }
    getQRData() {
        const bytes = this.getBytes();
        if (this.usage.type === DesignType_1.DesignType.Pro) {
            const parity = ByteUtils_1.calculateParity(bytes);
            return [
                QRData_1.QRData.fromBytes(ByteUtils_1.getByteSubset(bytes, 0, 540), 0, parity),
                QRData_1.QRData.fromBytes(ByteUtils_1.getByteSubset(bytes, 540, 540), 1, parity),
                QRData_1.QRData.fromBytes(ByteUtils_1.getByteSubset(bytes, 1080, 540), 2, parity),
                QRData_1.QRData.fromBytes(ByteUtils_1.getByteSubset(bytes, 1620, 540), 3, parity),
            ];
        }
        return [
            QRData_1.QRData.fromBytes(bytes),
        ];
    }
    // Import
    loadBytes(bytes) {
        this.loadMetadataBytes(bytes);
        this.colorData = ByteUtils_1.getByteSubset(bytes, 108, this.usage.colorDataLength);
    }
    loadMetadataBytes(data) {
        this.title = ByteUtils_1.getString(data, 0, 42);
        this.playerId = ByteUtils_1.getNumber(data, 42, 2);
        this.creator = ByteUtils_1.getString(data, 44, 18);
        this.sex = ByteUtils_1.getNumber(data, 62, 1);
        this.villageId = ByteUtils_1.getNumber(data, 64, 2);
        this.village = ByteUtils_1.getString(data, 66, 18);
        this.languageId = ByteUtils_1.getNumber(data, 84, 1);
        this.countryId = ByteUtils_1.getNumber(data, 86, 1);
        this.regionId = ByteUtils_1.getNumber(data, 87, 1);
        this.colorPalette.bytes = ByteUtils_1.getByteSubset(data, 88, 15);
        this.color = ByteUtils_1.getNumber(data, 103, 1);
        this.looks = ByteUtils_1.getNumber(data, 104, 1);
        this.usageId = ByteUtils_1.getNumber(data, 105, 1);
    }
    loadQRData(values) {
        if (values instanceof QRData_1.QRData) {
            values = [values];
        }
        // For normal QR data, we can just pass along the single QR code's data
        const normalData = values.find(d => d.type === DesignType_1.DesignType.Normal);
        if (normalData) {
            return this.loadBytes(normalData.bytes);
        }
        // For pro data, we need to apply metadata from the first code and color data from all four
        values.filter(d => d.type === DesignType_1.DesignType.Pro)
            .forEach(d => this.loadProQRData(d));
    }
    loadProQRData(data) {
        switch (data.index) {
            case 0:
                this.loadMetadataBytes(data.bytes);
                ByteUtils_1.setByteSubset(this.colorData, 0, ByteUtils_1.getByteSubset(data.bytes, 108, 432));
                break;
            case 1:
                ByteUtils_1.setByteSubset(this.colorData, 432, ByteUtils_1.getByteSubset(data.bytes, 0, 540));
                break;
            case 2:
                ByteUtils_1.setByteSubset(this.colorData, 972, ByteUtils_1.getByteSubset(data.bytes, 0, 540));
                break;
            case 3:
                ByteUtils_1.setByteSubset(this.colorData, 1512, ByteUtils_1.getByteSubset(data.bytes, 0, 536));
                break;
        }
    }
}
exports.Design = Design;
